/*
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * Swarco Traffic Systems GmbH
 * Entwicklung
 * Adolf-Dambach-Strae
 * D-76571 Gaggenau
 * Phone: +49 (0)7225-64-0
 * Fax: +49 (0)7225-64-300
 * mailto: office.sts@swarco.de
 */

package de.bsvrz.ste.stealgsba.algorithmen.lib;

import java.util.List;
import java.util.Vector;

import de.bsvrz.dav.daf.main.ClientDavConnection;
import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.config.Aspect;
import de.bsvrz.dav.daf.main.config.AttributeGroup;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.DataModel;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Diese Klasse implementiert ein Objekt vom Typ 'typ.sswSbaUmfeldDatenMessStelle'.<p>
 * 
 * Die Klasse dient zur Verwaltung von Daten einer Umfelddatenmessstelle.
 * 
 * ber Getter und Setter kann auf die Daten zugegriffen werden.
 *
 * @version $Revision: 1.2 $ / $Date: 2012/07/23 06:02:36 $ / ($Author: Asal $)
 *
 * @author Swarco Traffic Systems GmbH
 * @author Rolf Asal
 *
 */
public class UmfeldDatenMessStelle implements ClientReceiverInterface
{
	/**
   * Stellt Methoden fr die Debugausgabe zur Vefgung.
   */
	private Debug m_debug = Debug.getLogger();
	/**
	 * Aktuelle Verbindung zum Datenverteiler.
	 */
	private ClientDavConnection m_connection;
	/**
	 * Zugriff auf das aktuelle Datenmodell.
	 */
	private DataModel m_dataModel;
	/**
	 * Systemobjekt des Umfelddatenmessstelle.
	 */
	private SystemObject m_objekt;
	/**
	 * Systemobjekt der realen Umfelddatenmessstelle.
	 */
	private SystemObject m_objektReal;

	/**
	 * Attributgruppe fr den Datenempfang Nssestufe.
	 */
	private AttributeGroup m_atgUfdmsNaesseStufe;
	/**
	 * Attributgruppe fr den Datenempfang Ergebnis Tag/Nacht.
	 */
	private AttributeGroup m_atgHelligkeitErgebnisTagNacht;
	/**
	 * Aspekt fr den Datenempfang von Nssestufe.
	 */
	private Aspect m_aspKlssifizierung;
	/**
	 * Aspekt fr den Datenempfang Ergebnis Tag/Nacht.
	 */
	private Aspect m_aspZustand;
	/**
	 * Aktuelle Nssestufe der Umfelddatenmessstelle.
	 */
	private int m_naesseStufe;	
	/**
	 * Ergebnis der Tag/Nacht-Bestimmung.
	 */
	private ErgebnisTagNacht m_ergebnisTagNacht;	
	/**
	 * Liste der Umfelddatensensoren der Umfelddatenmessstelle.
	 */
	private List<SystemObject> m_umfeldDatenSensoren;
	
	/**
	 * Konstruktor mit bernahme der Parameter.<p>
	 * 
	 * @param connection Aktuelle Verbindung zum Datenverteiler.
	 * @param objekt Systemobjekt Messquerschnitt.
	 */
	public UmfeldDatenMessStelle(ClientDavInterface connection, SystemObject objekt)
	{
		m_connection = (ClientDavConnection)connection;
		m_dataModel = m_connection.getDataModel();
		
		m_objekt = objekt;
		m_objektReal = bestimmeUmfeldDatenMessStelleReal();
		m_umfeldDatenSensoren = bestimmeUmfeldDatenSensoren();
		
		m_atgUfdmsNaesseStufe = m_dataModel.getAttributeGroup("atg.sswSbaUfdmsNsseStufe");		
		m_aspKlssifizierung = m_dataModel.getAspect("asp.klassifizierung");
		m_atgHelligkeitErgebnisTagNacht = m_dataModel.getAttributeGroup("atg.sswSbaHelligkeitErgebnisTagNacht");
		m_aspZustand = m_dataModel.getAspect("asp.zustand");
	}

	/**
	 * Diese Methode bestimmt aus den Konfigurationsdaten die zugeordnete reale Umfelddatenmessstelle.
	 * @return Systemobjekt einer Umfelddatenmessstelle.
	 */
	private SystemObject bestimmeUmfeldDatenMessStelleReal()
	{
		if (m_objekt != null)
		{
			ConfigurationObject co = (ConfigurationObject)m_objekt;
		
			Data data = co.getConfigurationData(m_dataModel.getAttributeGroup("atg.sswSbaUmfeldDatenMessStelleReal"));
			if (data != null)
			{
				return data.getReferenceValue("UmfeldDatenMessStelleRealReferenz").getSystemObject();
			}
		
		}
		return null;
	}

	/**
	 * Diese Methode bestimmt alle Umfelddatensensoren der Umfelddatenmessstelle und gibt
	 * diese als Liste zurck.
	 * @return Liste mit Umfelddatensensoren.
	 */
	private List<SystemObject> bestimmeUmfeldDatenSensoren()
	{
		if (m_objektReal != null)
		{
			ConfigurationObject coUfdms = (ConfigurationObject)m_objektReal;
			
			return coUfdms.getNonMutableSet("UmfeldDatenSensoren").getElements();
		}
		
		return null;
	}
	/**
	 * Anmelden der angemeldeten Daten am Datenverteiler.
	 */
	public void datenDavAnmelden()
	{
		empfaengerAnmeldenObjekt(this, m_objekt, m_atgUfdmsNaesseStufe, m_aspKlssifizierung);
		empfaengerAnmeldenObjekt(this, m_objekt, m_atgHelligkeitErgebnisTagNacht, m_aspZustand);
	}
	/**
	 * Abmelden der angemeldeten Daten am Datenverteiler.
	 */
	public void datenDavAbmelden()
	{
		empfaengerAbmeldenObjekt(this, m_objekt, m_atgUfdmsNaesseStufe, m_aspKlssifizierung);
		empfaengerAbmeldenObjekt(this, m_objekt, m_atgHelligkeitErgebnisTagNacht, m_aspZustand);
	}
	
	/**
   * Fr ein Objekt Attributgruppe/Aspekt Kombination am Datenverteiler anmelden.
   * 
   * @param instanz Instanz.
   * @param objekt Objekt.
   * @param atg Attributgruppe.
   * @param asp Aspekt.
   * @return true = Anmeldung erfolgreich, false = nicht erfolgreich.
   */
  private boolean empfaengerAnmeldenObjekt(ClientReceiverInterface instanz, SystemObject objekt, AttributeGroup atg, Aspect asp)
  {
    DataDescription dd;
    boolean ok = false;

    m_debug.fine(("DatenAnmeldenObjekt " + atg.getPid() + " " + asp.getPid() + " fr " + objekt));
    
    try
    {  
    	dd = new DataDescription(atg, asp);

    	m_connection.subscribeReceiver(instanz, objekt, dd, ReceiveOptions.normal(), ReceiverRole.receiver());
        
    	ok = true;
    } 
    catch (DataNotSubscribedException e)
    {
      m_debug.error(("Daten nicht zum Empfangen angemeldet " + e.getMessage()));
    } 
    
    return (ok);
  }
	/**
   * Fr ein Objekt Attributgruppe/Aspekt Kombination am Datenverteiler anmelden.
   * 
   * @param instanz Instanz.
   * @param objekt Objekt.
   * @param atg Attributgruppe.
   * @param asp Aspekt.
   * @return true = Anmeldung erfolgreich, false = nicht erfolgreich.
   */
  private boolean empfaengerAbmeldenObjekt(ClientReceiverInterface instanz, SystemObject objekt, AttributeGroup atg, Aspect asp)
  {
    DataDescription dd;
    boolean ok = false;

    m_debug.fine(("DatenAbmeldenObjekt " + atg.getPid() + " " + asp.getPid() + " fr " + objekt));
    
  	try
		{
			dd = new DataDescription(atg, asp);

			m_connection.unsubscribeReceiver(instanz, objekt, dd);
			  
			ok = true;
		}
		catch (Exception e)
		{
			m_debug.error(("Daten konnten nicht abgemeldet werden " + e.getMessage()));
		}
    
    return (ok);
  }

	/* (non-Javadoc)
	 * @see de.bsvrz.dav.daf.main.ClientReceiverInterface#update(de.bsvrz.dav.daf.main.ResultData[])
	 */
	@Override
	public void update(ResultData[] results)
	{
		for (int i = 0; i < results.length; i++)
		{
			ResultData resultData = results[i];
			
			// Nssestufen
    	if (resultData.getDataDescription().getAttributeGroup().equals(m_atgUfdmsNaesseStufe))
    	{ 
//    		System.out.println("Nssestufe -> " + resultData);

    		if (resultData.hasData())
    		{
    			Data data = resultData.getData();
    			m_naesseStufe = data.getUnscaledValue("NsseStufe").intValue();
    		}
    		else
    		{    			
    			m_naesseStufe = -1; 
    		}
    	}

			// Ergebnis Tag/Nacht
    	if (resultData.getDataDescription().getAttributeGroup().equals(m_atgHelligkeitErgebnisTagNacht))
    	{ 
//    		System.out.println("ErgebnisTagNacht -> " + resultData);

    		if (m_ergebnisTagNacht == null)
    			m_ergebnisTagNacht = new ErgebnisTagNacht(resultData);
    		else
    			m_ergebnisTagNacht.setDaten(resultData);
    		
    		System.out.println("ErgebnisTagNacht -> " + m_ergebnisTagNacht);
    	}
		}		
	}

	/**
	 * Gibt die aktuelle Nssestufe zurck.
	 * @return aktuelle Nssestufe.
	 */
	public int getNaesseStufe()
	{
		return m_naesseStufe;
	}

	/**
	 * Gibt den Tag/Nacht-Status des bergebenen Umfelddatensensors zurck.
	 * 
	 * @param sensor Pid Umfelddatensensor.
	 * @return Tag/Nacht-Status.
	 */
	public String getStatusTagNacht(String sensor)
	{
		if (m_ergebnisTagNacht != null)
		{
			return m_ergebnisTagNacht.getStatusSensor(sensor);
		}
		
		return "Tag";
	}
	/**
	 * Gibt das Objekt zurck.
	 * @return Objekt
	 */
	public SystemObject getObjekt()
	{
		return m_objekt;
	}

	/**
	 * Gibt das Systemobjekt der realen Umfelddatenmessstelle zurck.
	 * @return Systemobjekt der realen Umfelddatenmessstelle.
	 */
	public SystemObject getObjektReal()
	{
		return m_objektReal;
	}

	/**
	 * Methode sucht in der Liste der Umfelddatensensoren den Sensor fr die Sichtweite
	 * und gibt diesen zurck. Ist kein Sensor fr die Sichtweite vorhanden, wird <code>null</code> zurckgegeben.
	 * @return Objekt Umfelddatensensor.
	 */
	public SystemObject getUmfeldDatenSensorSichtWeite()
	{
		if (m_umfeldDatenSensoren != null)
		{
			for (int i = 0; i < m_umfeldDatenSensoren.size(); i++)
			{
				SystemObject ufds = m_umfeldDatenSensoren.get(i);
				if (ufds.getType().getPid().equals("typ.ufdsSichtWeite"))
					return ufds;
			}			
		}
		return null;
	}
	/**
	 * Klasse zum Speichern des Ergebnisses zur Bestimmung von Tag/Nacht pro Umfelddatensensor.
	 *
	 * @version $Revision: 1.2 $ / $Date: 2012/07/23 06:02:36 $ / ($Author: Asal $)
	 *
	 * @author Swarco Traffic Systems GmbH
	 * @author Rolf Asal
	 *
	 */
	public class ErgebnisTagNacht
	{
		/**
		 * Ergebnis der Tag/Nacht-Bestimmung.
		 */
		private Vector<ErgebnisTagNachtSensor> m_ergebnisTagNacht = new Vector<ErgebnisTagNachtSensor>();
		
		/**
		 * Default-Konstruktor ohne weitere Funktion.
		 */
		public ErgebnisTagNacht()
		{		
		}
		
		/**
		 * Konstruktor mit bernahme der Daten.
		 * @param rdata zu bernehmender Datensatz.
		 */
		public ErgebnisTagNacht(ResultData rdata)
		{
			setDaten(rdata);
		}
		
		/**
		 * bernimmt die bergebenen Daten in die interne Struktur.
		 * @param rdata zu bernehmender Datensatz.
		 */
		public void setDaten(ResultData rdata)
		{
			m_ergebnisTagNacht.clear();
			
			if (rdata != null)
			{
				if (rdata.hasData())
				{
					Data data = rdata.getData();
					
					for (int i = 0; i < data.getArray("ErgebnisTagNacht").getLength(); i++)
					{
						Data element = data.getArray("ErgebnisTagNacht").getItem(i);
						if (element != null)
						{
							SystemObject sensor = element.getReferenceValue("Sensor").getSystemObject();
							String status = element.getScaledValue("Status").getValueText();
							
							addErgebnisTagNacht(sensor, status);
						}
					}
				}
			}			
		}
		
		/**
		 * Fgt zu der Ergebnisliste ein Ergebnis hinzu.
		 * @param sensor Umfelddatensenor.
		 * @param status Status Tag/Nacht.
		 */
		public void addErgebnisTagNacht(SystemObject sensor, String status)
		{
			if (m_ergebnisTagNacht != null)
			{
				ErgebnisTagNachtSensor etns = new ErgebnisTagNachtSensor();
				etns.setSensor(sensor);
				etns.setStatus(status);
				
				m_ergebnisTagNacht.add(etns);
			}
		}

		/* (non-Javadoc)
		 * @see java.lang.Object#toString()
		 */
		@Override
		public String toString() 
		{
			StringBuffer string = new StringBuffer();
			
			string.append(" " + m_ergebnisTagNacht);
			
			return string.toString();
	  }

		/**
		 * Gibt den Tag/Nacht-Status des Umfelddatensensor zurck.
		 * Konnte kein Status bestimmt werden, wird als Defaultwert 'Tag' zurckgegeben.
		 * 
		 * @param sensor PID Umfelddatensensor.
		 * @return Tag/Nacht-Status des Umfelddatensensor.
		 */
		public String getStatusSensor(String sensor)
		{
			String _status = "Tag";
			
			if (m_ergebnisTagNacht != null)
			{
				for (int i = 0; i < m_ergebnisTagNacht.size(); i++)
				{
					if (m_ergebnisTagNacht.get(i).getSensor().getPid().equals(sensor))
					{
						return m_ergebnisTagNacht.get(i).getStatus();
					}
				}
			}
			
			return _status;
		}
		/**
		 * Innere Klasse zum Speichern der Werte pro Umfelddatensensor.
		 *
		 * @version $Revision: 1.2 $ / $Date: 2012/07/23 06:02:36 $ / ($Author: Asal $)
		 *
		 * @author Swarco Traffic Systems GmbH
		 * @author Rolf Asal
		 *
		 */
		public class ErgebnisTagNachtSensor
		{
			/**
			 * Umfelddatensensor.
			 */
			private SystemObject m_sensor;
			/**
			 * Ergebnis Tag/Nacht.
			 */
			private String m_status;

			
			/**
			 * Default-Konstruktor ohne weitere Funktion.
			 */
			public ErgebnisTagNachtSensor()
			{				
			}
			
			/* (non-Javadoc)
			 * @see java.lang.Object#toString()
			 */
			@Override
			public String toString() 
			{
				StringBuffer string = new StringBuffer();
				
				string.append(" " + m_sensor + " Status -> " + m_status);
				
				return string.toString();
		  }

			/**
			 * Gibt den Umfelddatensensor zurck.
			 * @return Umfelddatensensor.
			 */
			public SystemObject getSensor()
			{
				return m_sensor;
			}
			/**
			 * Setzt den Umfelddatensensor.
			 * @param sensor zu setzender Umfelddatensensor.
			 */
			public void setSensor(SystemObject sensor)
			{
				m_sensor = sensor;
			}
			/**
			 * Gibt den Status Tag/Nacht zurck
			 * @return Status Tag/Nacht.
			 */
			public String getStatus()
			{
				return m_status;
			}
			/**
			 * Setrzt den Status Tag/Nacht.
			 * @param status zu setzender Status Tag/Nacht.
			 */
			public void setStatus(String status)
			{
				m_status = status;
			}
		}
	}

}
